﻿-- Action Bar Profiles
PAB = LibStub("AceAddon-3.0"):NewAddon("ProfilesActionBars", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

-- Minimap button
local icon = LibStub("LibDBIcon-1.0")
local bunnyLDB = LibStub("LibDataBroker-1.1"):NewDataObject("Bunnies!", {
	type = "launcher",
	icon = "Interface\\ICONS\\spell_nature_bloodlust",
	label = "AddonName",
	OnClick = function(self, button)
		if UnitAffectingCombat("player") then
			DEFAULT_CHAT_FRAME:AddMessage(L["|cff8888ffIceHUD|r: Combat lockdown restriction. Leave combat and try again."])
		end
		
		if button == "RightButton" or button == "LeftButton" then
			PAB:OpenMinimapMenu()
		end
		
	end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine("Profiles");
	end,
	
})

-- Configuration menu. Minimap
function PAB:OpenMinimapMenu(window)
	--DEFAULT_CHAT_FRAME:AddMessage("|cff8888ffIceHUD|r: Combat lockdown restriction. Leave combat and try again.")
	
	if not self.profilesmenu then
		self.profilesmenu = CreateFrame("Frame", "ProfilesMenu")
	end
	
	local profilesmenu = self.profilesmenu
	profilesmenu.displayMode = "MENU"
	local info = {}
	profilesmenu.initialize = function(self, level)
		if not level then return end
		wipe(info)
		if level == 1 then
			-- Title
			info.isTitle = 1
			info.text = "Synchronized profiles"
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			-- blank separator
			wipe(info)
			info.disabled = 1
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			---------------
			wipe(info)
			info.notCheckable = 1
			info.isNotRadio = true
			info.text = "-- Save config to current profile --"
			
			info.func = function() 
					PAB:PreSaveABProfile(PABdb:GetCurrentProfile())
			end
			
			UIDropDownMenu_AddButton(info, level)
			
			
			-- blank separator
			wipe(info)
			info.disabled = 1
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			-- Iterate available profiles
			for i,v in orderedPairs(PAB:ListProfiles(1)) do
				wipe(info)
				
				info.hasArrow = 1
				info.value = win
				info.isNotRadio = true
				info.func = function() 
					PAB:PreLoadABProfile(UIDROPDOWNMENU_MENU_VALUE)
				end
				
				
				
				info.checked = function() 
					
					if string.find(i,"\*") then
						return true
					end
				end
				
				info.text = string.gsub(i, "*", "")
				
				UIDropDownMenu_AddButton(info, level)
			end
			
			
			-- Available global (macros/keys and actionbar) profiles
			
			-- blank separator
			wipe(info)
			info.disabled = 1
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			
			------------
			wipe(info)
			info.isNotRadio = true
			info.text = "Mute status messages"
			
			info.func = function() 
					if PROFILES_TYST then
						PROFILES_TYST = nil
						PABdb.char.PROFILES_TYST = nil
						DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFProfiles: All status messages are now unmuted.")
					else
						PROFILES_TYST = true
						PABdb.char.PROFILES_TYST = true
						DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFProfiles: All status messages are now muted.")
					end
			end
			
			info.checked = function() 
				return PROFILES_TYST
			end
			UIDropDownMenu_AddButton(info, level)
			
			
			
			
			-- Close menu item
	        wipe(info)
	        info.text         = CLOSE
	        info.func         = function() CloseDropDownMenus() end
	        info.checked      = nil
	        info.notCheckable = 1
	        UIDropDownMenu_AddButton(info, level)
		elseif level == 2 then

			-- Chosen profile
			local profile_name = UIDROPDOWNMENU_MENU_VALUE
			
			-- Title
			info.isTitle = 1
			info.text = profile_name
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			
			wipe(info)
			info.text = "Activate"
			info.func = function() 
				PAB:PreLoadABProfile(profile_name)
			end
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			
			-- Comming soon(tm)
			-- wipe(info)
			-- info.text = "Rename"
			-- info.func = function() 
				-- DEFAULT_CHAT_FRAME:AddMessage("Profiles: All status messages are now muted.")
			-- end
			-- info.notCheckable = 1
			-- UIDropDownMenu_AddButton(info, level)
			
			-- Comming soon(tm)
			-- wipe(info)
			-- info.text = "Export"
			-- info.func = function() 
				-- DEFAULT_CHAT_FRAME:AddMessage("Profiles: All status messages are now muted.")
			-- end
			-- info.notCheckable = 1
			-- UIDropDownMenu_AddButton(info, level)
			
			-- blank separator
			wipe(info)
			info.disabled = 1
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = "Delete"
			info.func = function()
				PAB:PreDeleteProfile(profile_name)
			end
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			
		end
	end
	
	local x,y = GetCursorPosition(UIParent);
	ToggleDropDownMenu(1, nil, profilesmenu, "UIParent", x / UIParent:GetEffectiveScale() , y / UIParent:GetEffectiveScale())
end
----------------------- MINIMAP END -----------------------------------------------------------------------------------------


function PAB:OnInitialize()
   PABdb = LibStub("AceDB-3.0"):New("ProfilesActionBars",nil,"Restore [LOCKED]")
   MPglobaldb = LibStub("AceDB-3.0"):New("ProfilesActionBarsG",nil,"Restore [LOCKED]")
   icon:Register("Bunnies!", bunnyLDB, nil)
   PROFILES_TYST = PABdb.char.PROFILES_TYST
end

-- Export char profile to global
function PAB:ExportCharProfile(name)
	if PAB:ExistsProfile(name) then
		if PAB:ExistsProfile(name,1) then
			PAB:Echo("Overwritting global profile.."..name)
		end
		
		local orig_profile = PABdb:GetCurrentProfile()
		
		PABglobaldb:SetProfile(name)
		PABdb:SetProfile(name)
		
		-- MPglobaldb.profile = PABdb.profile
		for index, value in orderedPairs(PABdb.profile) do
		  PABglobaldb.profile[index] = value
		end
	
		PABdb:SetProfile(orig_profile)
		
		PAB:Echo("Profile exported, use import to load it: "..name)
		return 1
	else
		PAB:Echo("No such profile exists: "..name)
		return nil
	end
end

-- Import global profile to char
function PAB:ImportGlobalProfile(name)
	if PAB:ExistsProfile(name,1) then
		
		if PAB:ExistsProfile(name) then
			PAB:Echo("Overwritting profile.."..name)
		end
		
		-- TODO: SetProfile back to original
		PABdb:SetProfile(name)
		PABglobaldb:SetProfile(name)
		
		-- PABdb.profile = PABglobaldb.profile
		for index, value in orderedPairs(PABglobaldb.profile) do
		  PABdb.profile[index] = value
		end
		
		
		PAB:Echo("Profile imported: "..name)
		return 1
	else
		PAB:Echo("No such global profile exists: "..name)
		return nil
	end
end

function PAB:Echo(text)
if PROFILES_TYST then return end
   DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFProfiles: Action bars - "..text)
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function PAB:PreSaveABProfile(name)
	PAB:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function PAB:SaveABSProfile(name)

	-- Save a restore point if empty db
	if not PAB:ExistsProfile("Restore [LOCKED]") and not PABSAVERESTORE then 
		PABSAVERESTORE=1 
		PAB:SaveABSProfile("Restore [LOCKED]") 
		PABSAVERESTORE=nil
	end
	
	-- User trying to overwrite restore point!
	if name == "Restore [LOCKED]" and not PABSAVERESTORE then 
		PAB:Echo("This profile is locked at the first run and can only be loaded!")
		return nil
	end
	
	PABdb:SetProfile(name)

	-- Solving /mp save tank X 2
	if PAB:ExistsProfile(name) then PABdb:ResetProfile(name) end
	
   PAB:Echo("saving!")
   
	for actionID=1, 120 do
      
		local type_ab, id, subType, extraID = GetActionInfo(actionID)

		if type_ab and id then
			
			
			if type_ab == "flyout" then	-- Save name if flyout
				local flyOutName = GetFlyoutInfo(id)
				PABdb.profile[actionID] = {type_ab, flyOutName, actionID}
			
			else						-- Save spellID
				PABdb.profile[actionID] = {type_ab, id, actionID}
			end
		end
   end
   
   return name
end

-- This is an alias for ProfilesHub
function PAB:Save(...)
	PAB:SaveABSProfile(...)
end

-- ProfilesHub: Send message to all slaves that we are now switching profile
function PAB:PreLoadABProfile(name)

	
	-- No such profile saved
	if not PAB:ExistsProfile(name) then PAB:Echo(name.." profile not found.") return end

	-- In combat!
	if InCombatLockdown() then PAB:Echo("Cannot change macros in combat!") return end

	PAB:SendAceMsg("load",name) -- ProfilesHub
end

-- This function should not be called directly.
-- Should only be called by slave event!

function PAB:LoadABSProfile(name)

	-- No such profile saved
	if not PAB:ExistsProfile(name) then PAB:Echo(name.." profile not found.") return nil end
	
	-- In combat!
	if InCombatLockdown() then PAB:Echo("Cannot change macros in combat!") return end

	-- Save a restore point if empty db
	if not PAB:ExistsProfile("Restore [LOCKED]") and not PABSAVERESTORE then 
		PABSAVERESTORE=1 
		PAB:SaveABSProfile("Restore [LOCKED]")
		PABSAVERESTORE=nil
	end
	
	
	PABdb:SetProfile(name)

	PAB:Echo("Loading action from profile "..name.."!")
	
	-- Start fresh with nothing on the cursor
	ClearCursor()
	
	
	--Load Action bar profile!
	for index, keys in pairs(PABdb.profile) do
		
		if index ~= "numMacros" then
			
			-- Make sure we dont have anything on the cursor
			ClearCursor()

			-- No need to mess with duplicates
			local type_ab, action_id, _, _ = GetActionInfo(index)
			if not type_ab then type_ab = "99" end
			if not action_id then action_id = "99" end
			
			-- Ugly hack, i know
			local loaded_type_action_id = type_ab..action_id
			local saved_type_action_id = keys[1]..keys[2]
			
			
			-- Clear current spot if not duplicate with saved
			if loaded_type_action_id ~= saved_type_action_id then
			
				-- Do not touch macros!
				if type_ab ~= "macro" then PickupAction(index) end
			
				ClearCursor()
				
				
				if keys[1] == "spell" 		 then PickupSpell(keys[2]) end
				
				if keys[1] == "equipmentset" then PickupEquipmentSetByName(keys[2]) end
				
				if keys[1] == "item"         then 
				
					-- Do we have this item equiped ?
					if IsEquippedItem(keys[2]) then 
						PickupItem(keys[2])
					else
						-- Do we have item in inventory ?
						for bagNumber = -1, 11 do 
							--allow checking of bank slots, otherwise 0 to 4
							local nSlots = GetContainerNumSlots(bagNumber);
							for slotNumber = 1, nSlots do
								local id = GetContainerItemID(bagNumber, slotNumber);
								if id == keys[2] then 
									PickupItem(keys[2])
									break
								end
							end
						end
					end
					
				end
				
				if keys[1] == "companion"	 then
					local found_critter = nil
					-- Go through companions and see if we can find our saved SpellID
					for i=1,GetNumCompanions("CRITTER") do
						_,_,companionID = GetCompanionInfo("CRITTER", i)
						
						if keys[2] == companionID then
							PickupCompanion("CRITTER",i)
							found_critter = 1
							break
						end
					end
					
					if not found_critter then
						for i=1,GetNumCompanions("MOUNT") do
							_,_,companionID = GetCompanionInfo("MOUNT", i)
							
							if keys[2] == companionID then
								PickupCompanion("MOUNT",i)
								break
							end
						end
					end
				end
				
				if keys[1] == "flyout" then
					PickupSpellBookItem(PAB:GetSpellID_byName(keys[2]), BOOKTYPE_SPELL)
				end
				
				PlaceAction(index)
			end
		end
	end
	
	PAB:SendMessage("ABS_Profiles_load",name) -- ProfilesHub
	return name
end

-- This is an alias for ProfilesHub
function PAB:Load(...)
	PAB:LoadABSProfile(...)
end

-- Cache spells from book
-- Used mainly for flyout spells
PAB_spellCache = {}
function PAB:GetSpellID_byName(spellName)
   
   if PAB_spellCache[spellName] then
      return PAB_spellCache[spellName]
   else
      -- Build cache
      for book=1, MAX_SKILLLINE_TABS do
         local _, _, offset, numSpells = GetSpellTabInfo(book)
         
         for i=1, numSpells do
            local index = offset + i
            local spell, stance = GetSpellBookItemName(index, BOOKTYPE_SPELL)
            
            -- This way we restore the max rank of spells
            PAB_spellCache[spell] = index
            
            if( stance and stance ~= "" ) then
               PAB_spellCache[spell .. stance] = index
            end
         end
      end
      
      return PAB_spellCache[spellName]
   end
end

function PAB:ExistsProfile(name)
   for i, k in ipairs(PABdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end

function PAB:ListProfiles(silent)
    
	local PAB_profiles = {}
	for i,k in ipairs(PABdb:GetProfiles()) do
		if k == PABdb:GetCurrentProfile() then
			if not silent then PAB:Echo(' - ' .. k.." (current)") end
			PAB_profiles['* '..k] = 0
		else
			if not silent then PAB:Echo(' - ' .. k) end
			PAB_profiles[k] = 0
		end
	end
	
	return PAB_profiles
end

-- ProfilesHub
function PAB:PreDeleteProfile(name)
	PAB:SendAceMsg("delete",name) -- ProfilesHub
end

function PAB:DeleteProfil(name)

	-- User trying to overwrite restore point!
	if name == "Restore [LOCKED]" and not PABSAVERESTORE then 
		PAB:Echo("This profile cannot be deleted! It is locked at the first run and can only be loaded!")
		return nil
	end
	
	if (PAB:ExistsProfile(name)==nil)then
		PAB:Echo(name.." doesnt exist.")
		PAB:ListProfiles()
		return nil
	else
		if name == PABdb:GetCurrentProfile() then
			PABdb:SetProfile('Default')
		end
		PABdb:DeleteProfile(name)
		PAB:Echo(name.." deleted.")
		
		return name
	end
end

-- This is an alias for ProfilesHub
function PAB:Delete(...)
	PAB:DeleteProfil(...)
end
------------------------------